//
//  **************************************************************************
//
//  controller.h
//
//  (C) 2003 Bosco K. Ho 
//
//  
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//

#ifndef _controller_h
#define _controller_h

#include <list>
#include "wx/wx.h"
#include "myprot.h"


class Cmd;


class Controller
{
public:
  Controller(wxFrame* pParent, MyProtein*  pProtein): 
      mpParent(pParent), mpProtein(pProtein)
    { setMenuLabel(""); }

  void process(Cmd* pCmd); 
  
  void undo();
  bool canUndo() { return mCmdList.size() > 0; }
  void setUndoMarker(std::string name); 
  
  void setMenuLabel(std::string name);

  void clearMeasure() { mpProtein->mMeasure.clear(); };
  void toggleMeasure(AtomMarker a) { mpProtein->mMeasure.clickOnAtom(a); };

  void clearLoopClosure() { mpProtein->mLoopClosure.clear(); };
  void toggleLoopClosureRes(int i) { mpProtein->mLoopClosure.toggleRes(i); };
  void setLoopSolution(int i);

  void clearInteractions() { mpProtein->clearInteractions(); };
  void countInteractions() { mpProtein->countInteractions(); };
  
  void makeMeasurements() { mpProtein->makeMeasurements(); }
  
  void setPhiPsi(double phi, double psi);
  void setChi(int n, double chi);
  void setProPucker(double pucker);
  
  void loadProtein(std::string fname);
  void saveProtein(std::string fname);

  void setFocus(AtomMarker atom_marker) { mpProtein->mFocus.set(atom_marker); }
  void setInsertRes(int i) { mpProtein->mInsertRes = i; }

  void setSelect(int i, bool b) { mpProtein->mSelect.set(i, b); }
  void select(int i) { mpProtein->mSelect.set(i, !mpProtein->mSelect.isSelected(i)); }
  void resetSelect() { mpProtein->mSelect.reset(); }
  void selectAll();
  void copySelectionToClipboard();
  void deleteSelection();
  void insertClipboardInProtein(int i);
  void insertAlanine();

  void insertFromPdb(std::string fname);
  void copyToPdb(std::string fname);
  
  void makeAlaninePeptide();
  void makeAlphabetProtein();

  void buildHydrogen();
  void blockTerminals();
  void chargeTerminals();
  void mutate(int resKey);

  wxFrame* mpParent;
  MyProtein* mpProtein;
  std::list<Cmd*> mCmdList;
  MyProtein mClipboardProtein;
};

#endif
