//
//  **************************************************************************
//
//  protview.h 
//
//  (C) 2003 Bosco K. Ho 
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//

#ifndef _protview_h
#define _protview_h

#include <string>
#include <vector>
#include <list>
#include <sstream>     
#include <iomanip>

#include "wx/dcbuffer.h"
#include "wx/wx.h"
#include "wx/spinctrl.h"

#include "controller.h"
#include "myprot.h"
#include "graphic3d.h"
#include "widget.h"



#define PICK_FOCUS 0
#define PICK_RESIDUE 1
#define PICK_MEASURE 2
#define PICK_ANCHOR 3


// This class mediates interaction from program with 3d view and
// proteins, Converts the 3d vectors embeded in the protein
// structure mProtein into a series of 2d graphical object, stored
// in mGraphicsList, and the displays them. Obviously this conversion
// requires all kinds of filters in order to have different display
// option - requires a strategy for display

class ProtView : public Widget
{
public:
  ProtView(wxWindow* pParent, MyProtein*  p, Controller* c);

  void onRightMouseDown();
  void onRightMouseDrag();
  void onRightMouseUp();
  void onLeftMouseDown();
  void onLeftMouseDClick();
  void onLeftMouseDrag();
  void onLeftMouseUp();
  void clickAnchorRes(int i);

  void resetFocus(void);
  void resetFocusWithSameOrientation(void);
  void initializeFocus(void);

  void computeCameraFromFocus(void);

  void draw(wxBufferedPaintDC &dc);
  void renderHighlightInfoBox(wxBufferedPaintDC &dc);

  int atomColor(char element);
  bool isPosInSlab(Vector3d& test_pos);

  void renderProtein();

  void renderBond(Bond* rpBond);
  void renderFatBond(Bond* rpBond);
  void renderHbond(Bond* rpBond, int color);
  void renderDisulfideBond(Bond* rpBond);
  void renderBonds(BondList* pBonds, int style);

  void renderPeptidePlane(int i, int j);
  void renderResidueConnector(int i, int j);

  void renderAtomMarker(AtomMarker atom_marker);
  void renderAtomsInRes(int i);

  void renderHighlight(Vector3d& pos, int color);

  void renderMeasure();
  void renderAnchor();

  Controller* mpController;
  MyProtein*  mpProtein;
  Project3d   mProject3d;

  bool mIsBallStick;
  bool mIsCaTrace;
  bool mIsCaSidechain;
  bool mIsEverything;
  bool mIsBackbone;
  
  bool mIsHbonds;
  bool mIsWeakHbonds;
  bool mIsStericClash;

  bool mIsSlab;
  double mSlabThickness;
  bool mIsZDepth;

  bool mIsHydrogen;
  bool mIsPeptide;
  
  int mPickState;
  
  Vector3d mFocusPos;
  Vector3d mFocusUp;
  Vector3d mFocusAxis;
                
  Vector3d mCameraLook;
  Vector3d mCameraUp;
  Vector3d mCameraPos;
 
  AtomMarker mHighlight;
  int mHighlightZ;
  
  AtomMarker mPress;
           
  int mSaveMouseY;
  int mSaveMouseX;

  double mDisplayAtomRadius;
  double mDisplayLineWidth;
    
};


#endif
