//
//  ***************************************************************************
//
//  superpose.cpp
//  (C) 2004 Bosco K. Ho 
//
//  An object that calculates the optimal rotation between two sets of points
//  and calculates the rmsd. It is a C++ wrapper around the C files
//  rmsd.c and rmsd.h.
//
//  ***************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  ****************************************************************************
//

#include "superpose.h"
extern "C" {
  #include "rmsd.h" 
}



using namespace std;



void Superpose::addPoint(Vector3d* pRef, Vector3d* pMov)
{
  mRefList.push_back(pRef); 
  mMovList.push_back(pMov); 
}



void Superpose::clear()
{
  mRefList.clear();
  mMovList.clear();
}



void Superpose::calculateOptimalRotation()
{
  typedef double double3[3];

  int n_point = mRefList.size();

  double3* ref_buffer = (double3*) malloc(n_point * sizeof(double3));
  if (ref_buffer == NULL) 
    return;

  double3* mov_buffer = (double3*) malloc(n_point * sizeof(double3));
  if (mov_buffer == NULL) 
    return;
    
  for (int i=0; i<n_point; i++)
  {
    ref_buffer[i][0] = mRefList[i]->x();
    ref_buffer[i][1] = mRefList[i]->y();
    ref_buffer[i][2] = mRefList[i]->z();

    mov_buffer[i][0] = mMovList[i]->x();
    mov_buffer[i][1] = mMovList[i]->y();
    mov_buffer[i][2] = mMovList[i]->z();
  }

  double3 com, mov2ref;
  double U[3][3];

  calculate_rotation_rmsd(ref_buffer, mov_buffer, n_point, 
                          com, mov2ref, U, &mRmsd);

  free(ref_buffer);
  free(mov_buffer);

  mMatrix.makeIdentity();
  for (int i=0; i<3; i++)
    for (int j=0; j<3; j++)
      mMatrix.setElem(i, j, U[j][i]);
  
  Vector3d mov_com(com[0], com[1], com[2]);
  Vector3d mov_to_ref(mov2ref[0], mov2ref[1], mov2ref[2]);
  Vector3d disp = mov_com - mMatrix*mov_com + mov_to_ref;
  
  mMatrix = translate(disp) * mMatrix;
}

  
