//
//  **************************************************************************
//
//  widget.cpp
//  (C) 2003 Bosco K. Ho 
//
//  Basic definition of a screen/mouse widget. Simple widgets are
//  included: clickable, toggle and state buttons. As well, a mouse drawing
//  function is included that displays text.
//
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//


#include "widget.h"
#include "event.h"
#include <vector>
#include <algorithm>



///////////////////////////////////////////////////
// Base widget class
///////////////////////////////////////////////////



int Widget::getX()
{
  if (mX < 0)
  { 
    wxSize sz = mpParent->GetClientSize();
    return sz.GetWidth() + mX - getWidth() + 1;
  }
  else
  {
    return mX;
  }
}



int Widget::getY()
{
  if (mY < 0)
  { 
    wxSize sz = mpParent->GetClientSize();
    return sz.GetHeight() + mY - getHeight() + 1;
  }
  else
  {
    return mY;
  }
}




int Widget::getWidth()
{
  if (mWidth < 0)
  {
    wxSize sz = GetClientSize();  
    return sz.GetWidth();
  }
  else
    return mWidth;
    
}



int Widget::getHeight()
{
  if (mHeight < 0)
  {
    wxSize sz = GetClientSize();  
    return sz.GetHeight();
  }
  else
    return mHeight;
}



bool Widget::isInside (int x, int y)
{
  return ( (x >= getX()) && (x < (getX() + getWidth())) &&
           (y >= getY()) && (y < (getY() + getHeight())) );
}


