//
//  **************************************************************************
//
//  protein.h
//
//  (C) 2003 Bosco K. Ho 
//
//  Adapted from the The Biomolecule Toolkit (C) 2001, Eric Alm 
//  
//  **************************************************************************
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or (at
//  your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,  but
//  WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details. 
//  
//  You should have received a copy of the GNU Lesser General Public License 
//  along with this program; if not, write to the Free Software Foundation, 
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  **************************************************************************
//

#ifndef protein_h_
#define protein_h_

#include <cstdio>
#include <vector>
#include <set>
#include <map>
#include <iostream>

#include "residue.h"
#include "polymer.h"


class Protein : public Polymer 
{
  public:
    Protein(char chain_id = ' ', std::string name = "untitled") :
            Polymer(chain_id, name) {};
    Protein(const Protein& rhs);
    ~Protein() {}

    void    extractAngles();

    void    createRes(int i, int resKey);
    void    appendRes(int resKey);
    void    mutateRes(int i, int resKey);

    void    setPdbCode(std::string pdbCode) { mPdbCode = pdbCode; } 
    std::string getPdbCode() const  { return mPdbCode; } 

    void    blockTerminals();
    void    chargeTerminals();
    void    unblockNTerminal();
    void    unblockCTerminal();

    void    splice(int target, int seg1, int seg2);
    void    eraseResAndSplice(int leftRes, int rightRes);
    void    insertFragmentAndSplice(Protein& p, int i);
    
    void    extractProlineParam(int i);
    bool    projectProlineRing(int i, double phi);
    void    extractProlineChiAngles(int i);
    
    void    setBackbonePhi(int i, double phi);

    void    setPhi(int i, double phi);
    void    setPsi(int i, double psi);
    void    setChi(int i, int n, double chi);

    double& phi(int i)          { return mResList[i].param[0]; }
    double& psi(int i)          { return mResList[i].param[1]; }
    double& omega(int i)        { return mResList[i].param[2]; }
    double& chi(int i, int n)   { return mResList[i].param[3+n]; }

    void    setProPucker(int i, double pucker);
    double& proTheta(int i)     { return mResList[i].param[8]; }
    double& proPucker(int i)    { return mResList[i].param[10]; }
    double& proEta(int i)       { return mResList[i].param[11]; }
    double& proAlpha(int i)     { return mResList[i].param[12]; }
    double& proXi(int i)        { return mResList[i].param[13]; }
   
    void    buildAllHydrogens(int i);

    void    sproutBackboneFromPrev(int i);
    void    sproutBackboneFromNext(int i); 
    void    sproutEnds(int i);
    void    sproutSidechain(int i, int n);
    void    sproutWholeSidechain(int i);
    void    buildBackboneFromScratch(int i);
 
  private:
    std::string mPdbCode;
};



#endif

